\\ ---------- Methods for the computation of periods ----------

CY_Indicials(L,p)={
    L=polcoef(CY_Pointchange(L,p),0,z);
    local(indicials=factor(L));
    Set(vector(matsize(indicials)[1],X,[-polcoef(indicials[X,1],0,Theta)/polcoef(indicials[X,1],1,Theta),indicials[X,2]]))
};

CY_Pointchange(L,p)={
    local(tmp=1,Lnew=0);
    if(
        p!=oo,for(k=0,4,Lnew+=subst(polcoef(L,k,Theta),z,z+p)*tmp;tmp=(z+p)/z*(z*deriv(tmp,z)+tmp*Theta)),
        p==oo,Lnew=subst(subst(L,Theta,-Theta),z,1/z)
    );
    Lnew/z^(-poldegree(subst(lift(Lnew),z,1/z),z))
};

CY_Solve(L,p,ord,numval)={
    local(ind=CY_Indicials(L,p),indset=Set(apply(x->x[1],ind)));
    L=CY_Pointchange(L,p);
    local(f=vector(#ind,X,vector(ind[X][2],Y,0)));
    local(disc,num,c=0);
    for(X=1,#f,
        f[X][1]=z^ind[X][1]*(1+numval-numval);
        for(k=ind[X][1]+1,ord,
            for(Y=1,ind[X][2],
                disc=subst(polcoef(L,0,z),Theta,k);
                if(setsearch(indset,k)&&Y<ind[X][2],disc=subst(deriv(polcoef(L,0,z),Theta),Theta,k);c=1);
                if(setsearch(indset,k)==0||c==1,
                    num=sum(i=0,min(poldegree(L,z),k),sum(j=1,Y,subst(diffop(polcoef(L,i,z),[Theta],[1],j-1+c),Theta,k-i)/(j-1)!*polcoef(f[X][Y+1-j],k-i,z)));
                    f[X][Y]+=-num/disc*z^k
                );
                c=0;
            )
        )
    );
    f=apply(x->vector(#x,Y,sum(k=0,Y-1,x[Y-k]/k!*Log^k)),f);
    for(i=2,#f,f[1]=concat(f[1],f[i]));
    [f[1],p]
};

CY_Transmat(sol1,sol2,p)={
    local(A=matid(4),B=matid(4),p1,p2);
    A[,1]=sol1[1]~;
    B[,1]=sol2[1]~;
    for(i=2,4,
        A[,i]=(-z^2)^(sol1[2]==oo)*(deriv(A[,i-1],z)+deriv(A[,i-1],Log)/z);
        B[,i]=(-z^2)^(sol2[2]==oo)*(deriv(B[,i-1],z)+deriv(B[,i-1],Log)/z);
    );
    if(
        sol1[2]==oo,p1=1/p,
        True,p1=p-sol1[2]
    );
    if(
        sol2[2]==oo,p2=1/p,
        True,p2=p-sol2[2]
    );
    A=subst(subst(A,z,p1),Log,log(p1));
    B=subst(subst(B,z,p2),Log,log(p2));
    A*matsolve(B,matid(4))
};



\\ ---------- Methods for the computation of period polynomials ----------

PP_Integrals(f,k)={
		     local(fint=vector(k));
		     fint[1]=truncate(intformal(f/x));
		     for(i=2,k,fint[i]=intformal(fint[i-1]/x));
		     vector(k,X,fint[X]/(2*Pi*I)^X)~
		  };

PP_SL2(g,tau)=(g[1,1]*tau+g[1,2])/(g[2,1]*tau+g[2,2]);

PP_Boundary(P,g,n)=(g[2,1]*tau+g[2,2])^(-n)*subst(P,tau,PP_SL2(g,tau))-P;

PP_PerPol(fi,g)={
		   local(k=#fi,p1,p2,tmp1,tmp2,tau0=(I+g[1,1]*sign(g[2,1]))/max(1,abs(g[2,1])),tau1=PP_SL2(matsolve(g,matid(2)),tau0));
		   
		   p1=(tau-z)^(k-2);
		   p2=subst(bernpol(k-1,tau),tau,tau-z);
                   p2=p2-subst(p2,z,0);
		   p2=PP_Boundary(p2,g,2-k)/(k-1);
		   tmp1=sum(i=0,k-2,polcoef(p1,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau0^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))))-sum(i=0,k-2,polcoef(p1,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau1^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau1))));
		   tmp2=sum(i=0,k-1,polcoef(p2,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*(tau0-1)^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))))-sum(i=0,k-1,polcoef(p2,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau0^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))));
		   
		   (2*Pi*I)^(k-1)/(k-2)!*(tmp1+tmp2)
		};



\\ ---------- Computations ----------

default(parisize,"1G")
default(realprecision,"100")
default(format,"g.15")

ord = 1000;

L = Theta^4-2^8*z*(Theta+1/2)^4;
zStar = -1/2^8;
f0 = subst(CY_Solve(L,0,ord,1.0),Log,Log-Pi*I);
f0[1] *= [(2*Pi*I)^3,0,0,0;0,(2*Pi*I)^2,0,0;0,0,2*Pi*I,0;-8*zeta(3),0,0,1]~;
f1 = CY_Solve(L,zStar,ord,1.0);
transmat = CY_Transmat(f0,f1,zStar/2);
T = matrix(4,4);
T[,1] = f1[1]~;
for(i=1,3,T[,i+1] = (z+zStar)*deriv(T[,i],z));
T = transmat*subst(T,z,0);
print("Period matrix:");
printp(T);

ord = 8000;

[f] = mfeigenbasis(mfinit([32,3,-4],0));
f = Ser(subst(lift(mfcoefs(f,ord)),y,I));
fi = PP_Integrals(f,3);
fPol = PP_PerPol(fi,[129,-25;160,-31]);
[wfp,wfm] = ([5-4*I,17+9*I;-28+20*I,-88-48*I]^-1*[polcoef(fPol,0,tau),polcoef(fPol,1,tau)]~)~;
[wfp,wfm] = [real(wfp),I*imag(wfm)];
print("Periods of f: ",[wfp,wfm]);
print("Checking period polynomial of f: ",fPol-(wfp*((5-4*I)+tau*(-28+20*I))+wfm*((17+9*I)+tau*(-88-48*I))),"\n");

[g] = mfeigensearch([32,2]);
g = Ser(mfcoefs(g,ord));
gi = PP_Integrals(g,2);
gPol = PP_PerPol(gi,[27,-11;32,-13]);
wgp = real(gPol);
wgm = -I*imag(gPol);
print("Periods of g: ",[wgp,wgm]);
print("Checking period polynomial of g: ",gPol-(wgp-wgm),"\n");

h = mffrometaquo([32,16;16,-8]);
F = [0,0,0,0,0,0,0,0,0,0,0,25/2*I,0,-9,0,3*I,0,0,0,-1/2*I,0,-1];
F = mflinear(mfbasis([32,7,-4],1),matsolve(matconcat(apply(f->mfcoefs(f,21),mfbasis([32,7,-4],1))~)~,matconcat(F)~)[,1]);
F = Ser(mfcoefs(F,ord))/Ser(mfcoefs(h,ord));
Fi = PP_Integrals(F,3);
FPol = PP_PerPol(Fi,[129,-25;160,-31]);
[efp,efm] = ([5-4*I,17+9*I;-28+20*I,-88-48*I]^-1*[polcoef(FPol,0,tau),polcoef(FPol,1,tau)]~)~;
[efp,efm] = [real(efp),I*imag(efm)];
print("Quasiperiods of f: ",[efp,efm]);
print("Checking period polynomial of F: ",FPol-(efp*((5-4*I)+tau*(-28+20*I))+efm*((17+9*I)+tau*(-88-48*I))),"\n");

h = mffrometaquo([32,16;16,-8]);
[G] = mfsearch([32,6],[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0],1);
G = Ser(mfcoefs(G,ord))/Ser(mfcoefs(h,ord));
Gi = PP_Integrals(G,2);
GPol = PP_PerPol(Gi,[27,-11;32,-13]);
egp = real(GPol);
egm = -I*imag(GPol);
print("Quasiperiods of g: ",[egp,egm]);
print("Checking period polynomial of G: ",GPol-(egp-egm),"\n");

print("Checking Legendre relations: ",[matdet([wfp,efp;wfm,efm])-(2*Pi*I)^2*I,matdet([wgp,egp;wgm,egm])-(2*Pi*I)],"\n");

A = [48*I,-96,-48*I,0;24,0,-24,-48*I;14*I,-4,10*I,-24;5,12*I,7,2*I];
B = [4/3,-1/3,1/16,-1/192;0,0,-1/96,1/128;0,-1/12,1/24,-5/768;0,0,0,-1/1536];
print("Checking modularity of period matrix:");
printp(T-A*[wgp*wfp,wgp*efp,0,0;wgp*wfm,wgp*efm,0,0;0,0,egp*wfp,egp*efp;0,0,egp*wfm,egp*efm]*B);
