\\ ---------- Methods for the computation of Frobenius polynomials ----------

MUMSOL(L,ord)={
    local(f=[1,0,0,0]);
    L=matrix(4,poldegree(L,z)+1,X,Y,diffop(polcoef(L,Y-1,z),[Theta],[1],X-1));
    for(X=1,4,
        for(k=1,ord,
            f[X]-=sum(i=0,min(#L-1,k),sum(j=1,X,subst(L[j,i+1],Theta,k-i)/(j-1)!*polcoef(f[X-j+1],k-i,z)))/subst(L[1,1],Theta,k)*z^k
        );
        f[X]+=O(z^(ord+1))
    );
    f
};

BMATRIX(f)={
	local(df=z*deriv(f,z),ddf=z*deriv(df,z),dddf=z*deriv(ddf,z));
	[f[1],df[1],ddf[1],dddf[1];f[2],f[1]+df[2],2*df[1]+ddf[2],3*ddf[1]+dddf[2];f[3],f[2]+df[3],f[1]+2*df[2]+ddf[3],3*df[1]+3*ddf[2]+dddf[3];f[4],f[3]+df[4],f[2]+2*df[3]+ddf[4],f[1]+3*df[2]+3*ddf[3]+dddf[4]]
};

BMATRIX(f)={
	      local(df=z*deriv(f,z),ddf=z*deriv(df,z),dddf=z*deriv(ddf,z));
	      [f[1],df[1],ddf[1],dddf[1];f[2],f[1]+df[2],2*df[1]+ddf[2],3*ddf[1]+dddf[2];f[3],f[2]+df[3],f[1]+2*df[2]+ddf[3],3*df[1]+3*ddf[2]+dddf[3];f[4],f[3]+df[4],f[2]+2*df[3]+ddf[4],f[1]+3*df[2]+3*ddf[3]+dddf[4]]
	   };

FROB(L,c,ord,plist)={
    local(TRUNC=(M,n)->matrix(#M,#M,X,Y,M[X,Y]+O(z^(n+1))));
    local(f,b,binv,binvp,p,frob);
    f=MUMSOL(L,ord(plist[#plist]));
    b=BMATRIX(f);
    b*=[1,0,(26+32*z)/(1+256*z^2),(32*z-13312*z^2-8192*z^3)/(1+256*z^2)^2;0,1,(85)/(1+256*z^2),(26+32*z-36864*z^2+8192*z^3)/(1+256*z^2)^2;0,0,(85)/(1+256*z^2),(85-21760*z^2)/(1+256*z^2)^2;0,0,0,(85)/(1+256*z^2)];
    binv=matsolve(TRUNC(b,ord(1)),matid(4));
    print("Computed Frobenius matrix for the following primes:");
    for(k=1,#plist,
        p=plist[k];
        binvp=TRUNC(subst(binv,z,z^p),ord(p));
        frob=binvp*[1,0,0,0;0,p,0,0;0,0,p^2,0;c*ZETA3(p)*p^3,0,0,p^3]*TRUNC(b,ord(p));
        frob=truncate(lift(frob+O(p^4)*sum(i=0,ord(p),z^i)*matrix(4,4,X,Y,1)));
        plist[k]=[p,frob];
	print1(p," ");
    );
    print();
    plist
};

FROBPOL(frob,xlist)={
    local(p,tr1,tr2);
    p=frob[1];
    tr1=-trace(frob[2]);
    tr2=(tr1^2-trace(frob[2]^2))/(2*p);
    xlist=lift(apply(x->centerlift(subst([x,tr1+O(p^3),tr2+O(p^3)],z,teichmuller(x+O(p^3)))),xlist));
    xlist=apply(v->[v[1],if(v[2]<=4*p^(3/2),v[2],v[2]-p^3),if(v[3]<=6*p^2,v[3],v[3]-p^3)],xlist);
    apply(x->[x[1],1+x[2]*T+p*x[3]*T^2+p^3*x[2]*T^3+p^6*T^4],xlist)
};

\\ Numerical computation of p-adic zeta(3)
ZETA3(p) = {
  local(e=p^5,n=20);
  -(gamma(3*e+O(p^n))-3*gamma(2*e+O(p^n))+3*gamma(e+O(p^n))-1-(gamma(e+O(p^n))-1)^3)/(2*e^3) + O(p^10)
};



\\ ---------- Computations ----------

default(parisize,"1G")

L = (4294967296*z^8+1409286144*z^7+94371840*z^6+5505024*z^5+606208*z^4-21504*z^3+1440*z^2-84*z+1)*Theta^4+(17179869184*z^8+2818572288*z^7+201326592*z^6+55050240*z^5+1212416*z^4+129024*z^3+2688*z^2-168*z)*Theta^3+(25769803776*z^8+2281701376*z^7+283115520*z^6+30932992*z^5+819200*z^4+395264*z^3+3744*z^2-136*z)*Theta^2+(17179869184*z^8+872415232*z^7+201326592*z^6-4980736*z^5+212992*z^4+297984*z^3+2112*z^2-52*z)*Theta+(4294967296*z^8+134217728*z^7+50331648*z^6-6291456*z^5+98304*z^4+77824*z^3+384*z^2-8*z);
c = 1/4;
ord = x->3*x+10;
zStar = I/2^4;

primeList = primes(primepi(1000));
primeList = primeList[8..#primeList];

frob = FROB(L,c,ord,primeList);

points = vector(#frob,i,[frob[i][1],select(x->Mod(subst(z*polcoef(L,4,Theta)/(1+256*z^2)^2,z,x),frob[i][1])!=0,vector(frob[i][1],j,j-1))]);
print("\nComputed characteristic polynomials for the following primes:");
pols = vector(#frob,i,print1(frob[i][1]," ");[frob[i][1],FROBPOL(frob[i],points[i][2])]);
print();

factorizations = apply(v->[v[1],select(w->poldegree(factor(w[2])[1,1])<4,v[2])],pols);
print("\nNumbers of factorizations over Z:");
print(apply(v->[v[1],#v[2]],factorizations));

print("\nFactorization for polynomials of reductions of zStar:");
for(i=1,#pols,apply(v->print([v[1]+O(pols[i][1]),(w->matrix(#w~,#w,i,j,w[i,j]/if(j==1,polcoef(w[i,j],0,T),1)))(factor(v[2]))]),select(w->Mod((2^4*w[1])^2,pols[i][1])==-1,pols[i][2])));
