\\ ---------- Methods for computations with the monodromy action ----------

rankThreeIndices = [[1/2,1/2,1/2],[1/3,1/2,2/3],[1/4,1/2,3/4]];
rankThreeMonodromies = [[[1,0,0;1,1,0;1/2,1,1],[0,0,-8;0,1,0;-1/8,0,0]],[[1,0,0;1,1,0;1/2,1,1],[0,0,-6;0,1,0;-1/6,0,0]],[[1,0,0;1,1,0;1/2,1,1],[0,0,-4;0,1,0;-1/4,0,0]]];

decomposition(cycle,v,monodromies,dim) = {
	local(result);
	result = vector(dim,i,[0,0,0]);
	while(#cycle>0,
		if(cycle[1]>0,
			result += vector(dim,i,if(i==cycle[1],v,[0,0,0])),
			result += vector(dim,i,if(i==-cycle[1],-v*monodromies[abs(cycle[1])]^-1,[0,0,0]))
		);
		v = v*monodromies[abs(cycle[1])]^sign(cycle[1]);
		cycle = cycle[2..#cycle];
	);
	result
};

constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods) = {
	local(dim,monodromies,newMonodromies,mMum,mCon,boundaryTerms,rowSpace,l,tmp,wordLength);
	dim = #cyclesPushforward;
	for(i=1,3,if(rankThreeData==rankThreeIndices[i],monodromies=rankThreeMonodromies[i]));
	newMonodromies = apply(cycle->vecprod(apply(i->monodromies[abs(i)]^sign(i),cycle)),cyclesPushforward);
	for(i=1,#newMonodromies,newMonodromies[i]*=monodromySquareRoot[i]);
	mMum = vector(3*dim,i,vector(3*dim,j,j==i));
	mMum = apply(v->vector(#v/3,i,v[1+(i-1)*3..3*i]),mMum);
	mMum = matrix(#mMum,#mMum,i,j,concat(mMum[i])*concat(sum(k=1,dim,decomposition(cycleActionMumMonodromy[k],mMum[j][k]*monodromies[1],newMonodromies,dim)))~);
	mCon = vector(3*dim,i,vector(3*dim,j,j==i));
	mCon = apply(v->vector(#v/3,i,v[1+(i-1)*3..3*i]),mCon);
	mCon = matrix(#mCon,#mCon,i,j,concat(mCon[i])*concat(sum(k=1,dim,decomposition(cycleActionConMonodromy[k],mCon[j][k],newMonodromies,dim)))~);
	print(" -Checking action of Mum monodromy: ",vector(dim,i,monodromies[1]*vecprod(apply(j->newMonodromies[abs(j)]^sign(j),cycleActionMumMonodromy[i]))*monodromies[1]^-1==newMonodromies[i]));
	print(" -Checking action of Con monodromy: ",vector(dim,i,vecprod(apply(j->newMonodromies[abs(j)]^sign(j),cycleActionConMonodromy[i]))==newMonodromies[i]));
	boundaryTerms = (v->sum(i=1,dim,v[3*(i-1)+1..3*i]~*(newMonodromies[i]-1)));
	l = matconcat(concat(decomposition(cycleHolomorphicPeriod,[1,0,0],newMonodromies,dim))~);
	tmp = 0;
	wordLength = 0;
	while(#l>tmp,
		print(" -Words of length up to ",wordLength," give dimension ",#l,".");
		wordLength++;
		tmp = #l;
		l = matconcat([l,matconcat(vector(#l,i,mMum*l[,i])),matconcat(vector(#l,i,mMum^-1*l[,i])),matconcat(vector(#l,i,mCon*l[,i])),matconcat(vector(#l,i,mCon^-1*l[,i]))]);
		l = matimage(l);
	);
	l = matconcat([l,matconcat(apply(v->matconcat(concat(decomposition(v[1],v[2],newMonodromies,dim))~),vanishingPeriods))]);
	l = matimage(l);
	print(" -Adding vanishing periods gives space of dimension ",#l,".");
	l = l*matker(matrix(3*dim,#l,i,j,if(i<=3,l[i,j],0)));
	print(" -Restricting to paths that don't get pinched gives space of dimension ",#l,".");
	print(" -Checking boundary terms: ",vector(#l,i,boundaryTerms(l[,i])==[0,0,0]),"\n");
};


\\ ---------- Computations ----------

default(parisize,"1G")

print("Computations for [1/2,1/2,1/2,1/2], [1/2,1/2,1/2] and (k,l,beta)=(1,1,1):");
rankThreeData = [1/2,1/2,1/2];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-1,4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1],[-2,1],[1,1],[2,1]];
monodromySquareRoot = [1,1,1,1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/4,1/3,2/3,3/4], [1/3,1/2,2/3] and (k,l,beta)=(1,1,1/2):");
rankThreeData = [1/3,1/2,2/3];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-1,4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1],[-2,1],[1,1],[2,1]];
monodromySquareRoot = [-1,-1,-1,-1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/4,1/2,1/2,3/4], [1/4,1/2,3/4] and (k,l,beta)=(1,1,1):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-1,4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1],[-2,1],[1,1],[2,1]];
monodromySquareRoot = [1,1,1,1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/5,2/5,3/5,4/5], [1/4,1/2,3/4] and (k,l,beta)=(1,4,1):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[6],[-6,5,7],[7],[-7,-5,-4,1]];
cycleActionConMonodromy = [[4,5,7,-1,2],[2],[3],[4],[5],[6],[7]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1,1,1,1],[1,1,-2,1,1],[1,1,1,1,1],[1,-2,-1],[1,2,1,2,1,2,1],[-2],[-1,2,1]];
monodromySquareRoot = [1,1,1,1,1,1,1];
vanishingPeriods = [[[2,-7,-5,-4],[1,0,0]],[[2,-7,-5,-4],[0,1,0]],[[2,-7,-5,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/3,1/3,2/3,2/3], [1/3,1/2,2/3] and (k,l,beta)=(2,1,1):");
rankThreeData = [1/3,1/2,2/3];
cycleActionMumMonodromy = [[1],[1,-3,4],[1,-2,4],[4],[-1,5,1]];
cycleActionConMonodromy = [[5,-1,2],[2],[3],[4],[5]];
cycleHolomorphicPeriod = [1,-4];
cyclesPushforward = [[1],[-2,1],[1,2,1,1],[1,1,1],[2,1]];
monodromySquareRoot = [1,1,1,1,1];
vanishingPeriods = [[[2,-5],[1,0,0]],[[2,-5],[0,1,0]],[[2,-5],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/4,1/4,3/4,3/4], [1/4,1/2,3/4] and (k,l,beta)=(1,1,1/2):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-1,4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1],[-2,1],[1,1],[2,1]];
monodromySquareRoot = [-1,-1,-1,-1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/3,1/2,1/2,2/3], [1/3,1/2,2/3] and (k,l,beta)=(1,1,1):");
rankThreeData = [1/3,1/2,2/3];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-1,4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1],[-2,1],[1,1],[2,1]];
monodromySquareRoot = [1,1,1,1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/6,1/2,1/2,5/6], [1/4,1/2,3/4] and (k,l,beta)=(1,2,1/2):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,2,-1,3],[3],[-5,1],[-4,1]];
cycleActionConMonodromy = [[4,-1,2],[2],[3],[4],[5]];
cycleHolomorphicPeriod = [1,-3];
cyclesPushforward = [[1,1],[1,-2,1],[1,1,1],[1,2,1],[-2]];
monodromySquareRoot = [-1,-1,-1,-1,1];
vanishingPeriods = [[[2,-4],[1,0,0]],[[2,-4],[0,1,0]],[[2,-4],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/6,1/3,2/3,5/6], [1/3,1/2,2/3] and (k,l,beta)=(2,1,1/2):");
rankThreeData = [1/3,1/2,2/3];
cycleActionMumMonodromy = [[1],[1,-3,4],[1,-2,4],[4],[-1,5,1]];
cycleActionConMonodromy = [[5,-1,2],[2],[3],[4],[5]];
cycleHolomorphicPeriod = [1,-4];
cyclesPushforward = [[1],[-2,1],[1,2,1,1],[1,1,1],[2,1]];
monodromySquareRoot = [-1,-1,-1,-1,-1];
vanishingPeriods = [[[2,-5],[1,0,0]],[[2,-5],[0,1,0]],[[2,-5],[0,0,1]],[[5],[1,3,6]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/8,3/8,5/8,7/8], [1/4,1/2,3/4] and (k,l,beta)=(2,2,1/2):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,4],[1,-2,4],[4],[-6,1],[-5,1]];
cycleActionConMonodromy = [[6,-1,2],[2],[3],[4],[5],[6]];
cycleHolomorphicPeriod = [1,-4];
cyclesPushforward = [[1,1],[1,-2,1],[1,1,2,1,1],[1,1,1,1],[-2],[1,2,1]];
monodromySquareRoot = [-1,-1,-1,-1,1,-1];
vanishingPeriods = [[[2,-6],[1,0,0]],[[2,-6],[0,1,0]],[[2,-6],[0,0,1]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/6,1/4,3/4,5/6], [1/4,1/2,3/4] and (k,l,beta)=(2,1,1/2):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,-3,4],[1,-2,4],[4],[-1,5,1]];
cycleActionConMonodromy = [[5,-1,2],[2],[3],[4],[5]];
cycleHolomorphicPeriod = [1,-4];
cyclesPushforward = [[1],[-2,1],[1,2,1,1],[1,1,1],[2,1]];
monodromySquareRoot = [-1,-1,-1,-1,-1];
vanishingPeriods = [[[2,-5],[1,0,0]],[[2,-5],[0,1,0]],[[2,-5],[0,0,1]],[[5],[1,2,4]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);

print("Computations for [1/10,3/10,7/10,9/10], [1/4,1/2,3/4] and (k,l,beta)=(4,1,1/2):");
rankThreeData = [1/4,1/2,3/4];
cycleActionMumMonodromy = [[1],[1,5],[1,-2,3,5],[2,-1],[-5,-3,-4,6],[6],[-1,7,1]];
cycleActionConMonodromy = [[7,-1,2],[2],[3],[4],[5],[6],[7]];
cycleHolomorphicPeriod = [1,-6];
cyclesPushforward = [[1],[-2,1],[1,2,1,2,1,2,1,1],[1,-2,-1],[-1,-1,-2,1,1],[1,1,1,1,1],[2,1]];
monodromySquareRoot = [-1,-1,-1,1,1,-1,-1];
vanishingPeriods = [[[2,-7],[1,0,0]],[[2,-7],[0,1,0]],[[2,-7],[0,0,1]],[[7],[1,2,4]]];
constructCycles(rankThreeData,cycleActionMumMonodromy,cycleActionConMonodromy,cycleHolomorphicPeriod,cyclesPushforward,monodromySquareRoot,vanishingPeriods);
