\\ ---------- Methods for the computation of periods ----------

CY_Indicials(L,p)={
    L=polcoef(CY_Pointchange(L,p),0,z);
    local(indicials=factor(L));
    Set(vector(matsize(indicials)[1],X,[-polcoef(indicials[X,1],0,Theta)/polcoef(indicials[X,1],1,Theta),indicials[X,2]]))
};

CY_Pointchange(L,p)={
    local(tmp=1,Lnew=0);
    if(
        p!=oo,for(k=0,4,Lnew+=subst(polcoef(L,k,Theta),z,z+p)*tmp;tmp=(z+p)/z*(z*deriv(tmp,z)+tmp*Theta)),
        p==oo,Lnew=subst(subst(L,Theta,-Theta),z,1/z)
    );
    Lnew/z^(-poldegree(subst(lift(Lnew),z,1/z),z))
};

CY_Solve(L,p,ord,numval)={
    local(ind=CY_Indicials(L,p),indset=Set(apply(x->x[1],ind)));
    L=CY_Pointchange(L,p);
    local(f=vector(#ind,X,vector(ind[X][2],Y,0)));
    local(disc,num,c=0);
    for(X=1,#f,
        f[X][1]=z^ind[X][1]*(1+numval-numval);
        for(k=ind[X][1]+1,ord,
            for(Y=1,ind[X][2],
                disc=subst(polcoef(L,0,z),Theta,k);
                if(setsearch(indset,k)&&Y<ind[X][2],disc=subst(deriv(polcoef(L,0,z),Theta),Theta,k);c=1);
                if(setsearch(indset,k)==0||c==1,
                    num=sum(i=0,min(poldegree(L,z),k),sum(j=1,Y,subst(diffop(polcoef(L,i,z),[Theta],[1],j-1+c),Theta,k-i)/(j-1)!*polcoef(f[X][Y+1-j],k-i,z)));
                    f[X][Y]+=-num/disc*z^k
                );
                c=0;
            )
        )
    );
    f=apply(x->vector(#x,Y,sum(k=0,Y-1,x[Y-k]/k!*Log^k)),f);
    for(i=2,#f,f[1]=concat(f[1],f[i]));
    [f[1],p]
};

CY_Transmat(sol1,sol2,p)={
    local(A=matid(4),B=matid(4),p1,p2);
    A[,1]=sol1[1]~;
    B[,1]=sol2[1]~;
    for(i=2,4,
        A[,i]=(-z^2)^(sol1[2]==oo)*(deriv(A[,i-1],z)+deriv(A[,i-1],Log)/z);
        B[,i]=(-z^2)^(sol2[2]==oo)*(deriv(B[,i-1],z)+deriv(B[,i-1],Log)/z);
    );
    if(
        sol1[2]==oo,p1=1/p,
        True,p1=p-sol1[2]
    );
    if(
        sol2[2]==oo,p2=1/p,
        True,p2=p-sol2[2]
    );
    A=subst(subst(A,z,p1),Log,log(p1));
    B=subst(subst(B,z,p2),Log,log(p2));
    A*matsolve(B,matid(4))
};



\\ ---------- Computations ----------

default(parisize,"1G")
default(realprecision,"100")
default(format,"g.15")

ord = 500;

heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi) = {
	local(L,f0,f1,T,wm,am,b,c);
	L = Theta^4-z/mu*(Theta+a1)*(Theta+a2)*(Theta+a3)*(Theta+a4);
	f0 = CY_Solve(L,0,ord,1.0);
	f0[1] *= [(2*Pi*I)^3,0,0,0;0,(2*Pi*I)^2,0,0;c2D*(2*Pi*I)^3/24,(kappa%2)/2*(2*Pi*I)^2,-kappa*2*Pi*I,0;chi*zeta(3),c2D*(2*Pi*I)^2/24,0,kappa]~;
	f1 = CY_Solve(L,mu,ord,1.0);
	f1[1] *= [0,1/mu*log(-mu),-1/mu,0;1,0,0,0;0,0,0,(-1/mu)^2;0,-1/mu,0,0]~;
	T = CY_Transmat(f0,f1,mu/2);
	[wm,am,b,c] = [I*imag(T[3,2]),I*imag(T[3,4]),T[1,2],T[1,4]];
	[1+matdet([b,c;wm,am])/(2*Pi*I*sqrt(kappa)*wm),wm]
};

twist(f,d) = {
	local(N,mf,fTwisted);
	N = mfparams(f)[1];
	N = lcm([N,N*d,d^2]);
	mf = mfinit([N,4],1);
	[fTwisted] = mfsearch([N,4],vector(mfsturm(mf)+1,i,kronecker(d,i-1)*mfcoef(f,i-1)),1);
	fTwisted = mftonew(mf,fTwisted);
	fTwisted = apply(v->v[3],fTwisted);
	forprime(p=2,oo,if(N%p!=0,fTwisted=select(g->mfcoef(g,p)==kronecker(d,p)*mfcoef(f,p),fTwisted));if(#fTwisted==1,break));
	fTwisted[1]
};

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/2,1/2,1/2,1/2,1/2^8,16,64,-128];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-3*log(2)));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/4,1/3,2/3,3/4,1/(2^6*3^3),6,48,-156];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([9,4]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-6*log(2)-3*log(3)-144*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/4,1/2,1/2,3/4,1/2^10,8,56,-176];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([16,4]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-6*log(2)-128*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/5,2/5,3/5,4/5,1/5^5,5,50,-200];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([25,4],[[2,1]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-5/3*log(5)-125/6*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/3,1/3,2/3,2/3,1/3^6,9,54,-144];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([27,4],[[2,-3]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-3*log(3)-243/2*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/4,1/4,3/4,3/4,1/2^12,4,40,-144];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([32,4],[[3,-8]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-5*log(2)-64*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/3,1/2,1/2,2/3,1/(2^4*3^3),12,60,-144];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([36,4]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-4*log(2)-3*log(3)-288*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/6,1/2,1/2,5/6,1/(2^8*3^3),4,52,-256];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([72,4],[[5,-14]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-3*log(2)-3*log(3)-216*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/6,1/3,2/3,5/6,1/(2^4*3^6),3,42,-204];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([108,4],[[5,-9]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-4*log(2)-3*log(3)-108*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/8,3/8,5/8,7/8,1/2^16,2,44,-296];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([128,4],[[3,-2],[5,6]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-7*log(2)-128*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/6,1/4,3/4,5/6,1/(2^10*3^3),2,32,-156];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([144,4],[[5,16]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-6*log(2)-3*log(3)-144*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/10,3/10,7/10,9/10,1/(2^8*5^5),1,34,-288];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([200,4],[[3,1]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-3*log(2)-5/3*log(5)-250/3*leadingCoefficient));

[a1,a2,a3,a4,mu,kappa,c2D,chi] = [1/6,1/6,5/6,5/6,1/(2^8*3^6),1,22,-120];
[height,wm] = heightAndPeriod(a1,a2,a3,a4,mu,kappa,c2D,chi);
[f] = mfeigensearch([216,4],[[5,1]]);
fTwisted = twist(f,nfdisc(x^2-kappa));
leadingCoefficient = 2*Pi*I*sqrt(kappa)*lfun(lfunmf(mfinit(fTwisted),fTwisted),2,1)/wm;
print("Checking height-identity for indices ",[a1,a2,a3,a4],": ",height-(-3*log(2)-3*log(3)-54*leadingCoefficient));
